﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;
	using System.Collections;
	using System.Collections.Generic;
	
	public enum ClasterEventType {
		Added = 0,
		Removed = 1
	}

	public class ClasterCollection: TemplatedList<ClasterCollection, Claster> {

		internal ClasterCollection() : base() { }

		public Claster this[ string iSymbol ] {
			get {
				Claster _Next = this.First;
				Claster _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item.Symbol == iSymbol )
						return _Item;
				}
				return null;
			}
		}
		
		public Claster this[ Context iContext ] {
			get {
				Claster _Next = this.First;
				Claster _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item == iContext.Claster )
						return _Item;
				}
				return null;
			}
		}

		public Claster Append( Context iContext, string iSymbol, int iDigits, int iSpread, double iPoint ) {
			if ( iContext == null )
				throw new ArgumentNullException( "iContext" );
			if ( iContext.Claster != null )
				throw new ArgumentException( "Claster is defined", "iContext" );
			if ( iSymbol == null )
				throw new ArgumentNullException( "iSymbol" );
			Claster _Item = this[ iSymbol ];
			if ( _Item == null ) {
				_Item = AddFinal( new Claster( iSymbol, iDigits, iSpread, iPoint ) );
			} else {
				_Item.Digits = iDigits;
				_Item.Spread = iSpread;
				_Item.Point = iPoint;
			}
			_Item.Session.Append( iContext );
			return _Item;
		}

		public Claster Remove( Context iContext ) {
			if ( iContext == null )
				throw new ArgumentNullException( "iContext" );
			if ( iContext.Claster == null )
				throw new ArgumentException( "Claster not defined", "iContext" );
			Claster _Item = this[ iContext ];
			_Item.Session.Remove( iContext );
			return _Item;
		}

		public Claster Remove( string iSymbol ) {
			if ( iSymbol == null )
				throw new ArgumentNullException( "iSymbol" );
			Claster _Item = this[ iSymbol ];
			if ( _Item != null ) {
				Remove( _Item );
			}
			return _Item;
		}

	}

	public class Claster: TemplatedItem<ClasterCollection, Claster> {

		private string mSymbol;
		private SessionCollection mSession;
		private ClasterTickCollection mTicks;

		public string Symbol {
			get {
				return mSymbol;
			}
		}
		
		public SessionCollection Session {
			get {
				return mSession;
			}
		}
		public ClasterTickCollection Ticks {
			get {
				return mTicks;
			}
		}

		private int mDigits;
		private int mSpread;
		private double mPoint;

		private ClasterMono mUp;
		private ClasterMono mDown;

		public ClasterMono Up {
			get {
				return mUp;
			}
			set {
				mUp = value;
			}
		}

		public ClasterMono Down {
			get {
				return mDown;
			}
			set {
				mDown = value;
			}
		}
		
		public int Digits {
			get {
				return mDigits;
			}
			set {
				mDigits = value;
			}
		}

		public int Spread {
			get {
				return mSpread;
			}
			set {
				mSpread = value;
			}
		}

		public double Point {
			get {
				return mPoint;
			}
			set {
				mPoint = value;
			}
		}

		internal Claster( string iSymbol, int iDigits, int iSpread, double iPoint )
			: base() {
			mSymbol = iSymbol;
			mSession = new SessionCollection( this );
			mTicks = new ClasterTickCollection( this, -1 );
			mDigits = iDigits;
			mSpread = iSpread;
			mPoint = iPoint;
			mUp = null;
			mDown = null;
			mDiagramTickHeight = 10;
			mDiagramTickWidth = 1;
		}

		public override void OnAdded( ClasterCollection iList ) {
			Global.Compare.AppendClaster( this );
			base.OnAdded( iList );
		}
		

		public override void OnRemoved( ClasterCollection iList ) {
			Global.Compare.RemoveClaster( this );
			Session.Clear();
			Ticks.Clear();
			base.OnAdded( iList );
		}
		
		private int mDiagramTickWidth;
		
		public int DiagramTickWidth {
			get {
				return mDiagramTickWidth;
			}
			set {
				mDiagramTickWidth = value;
			}
		}

		private int mDiagramTickHeight;

		public int DiagramTickHeight {
			get {
				return mDiagramTickHeight;
			}
			set {
				mDiagramTickHeight = value;
			}
		}

	}

}