﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class ClasterTickCollection: TemplatedList<ClasterTickCollection, ClasterTick> {

		private Claster mClaster;
		public Claster Claster {
			get {
				return mClaster;
			}
		}
		
		internal int mMaxCount = 65535;
		
		public int MaxCount {
			get {
				return mMaxCount;
			}
		}

		internal ClasterTickCollection( Claster iClaster, int iMaxCount )
			: base() {
			mClaster = iClaster;
			if ( iMaxCount > 0 && iMaxCount < 1000000 )
				mMaxCount = iMaxCount;
		}

		public ClasterTick AddValue( DateTime iUtc, double iBid, DateTime iSrv ) {
			while ( Count >= MaxCount ) {
				Remove( First );
			}
			return AddFinal( new ClasterTickValue( iUtc, iSrv, iBid ) );
		}

		public ClasterTick AddSpace( DateTime iUtc ) {
			while ( Count >= MaxCount ) {
				Remove( First );
			}
			if ( Final is ClasterTickSpace ) {
				return Final;
			} else {
				return AddFinal( new ClasterTickSpace( iUtc ) );
			}
		}

	}

	public class ClasterTick: TemplatedItem<ClasterTickCollection, ClasterTick> {

		private DateTime mCalledStamp;

		public DateTime CalledStamp {
			get {
				return mCalledStamp;
			}
		}

		public double Ask {
			get {
				return Bid + ( List.Claster.Point * List.Claster.Spread );
			}
		}

		public virtual double Bid {
			get {
				throw new Exception( "Undefined property" );
			}
		}

		public double AskInt {
			get {
				return ( Bid + ( List.Claster.Point * List.Claster.Spread ) ) * ( Math.Pow( 10, List.Claster.Digits ) );
			}
		}

		public double BidInt {
			get {
				return Bid * ( Math.Pow( 10, List.Claster.Digits ) );
			}
		}
		
		public virtual DateTime ServerStamp {
			get {
				throw new Exception( "Undefined property" );
			}
		}

		protected ClasterTick( DateTime iCalledStamp )
			: base() {
			mCalledStamp = iCalledStamp;
		}

	}

	public class ClasterTickSpace: ClasterTick {

		public ClasterTickSpace( DateTime iCalledStamp )
			: base( iCalledStamp ) {
			
		}
		
	}

	public class ClasterTickValue: ClasterTick {

		private DateTime mServerStamp;
		private double mBid;

		public ClasterTickValue( DateTime iCalledStamp, DateTime iServerStamp, double iBid )
			: base( iCalledStamp ) {
			mServerStamp = iServerStamp;
			mBid = iBid;
		}

		public override double Bid {
			get {
				return mBid;
			}
		}

		public override DateTime ServerStamp {
			get {
				return mServerStamp;
			}
		}
		
	}

}