﻿/******************************************************************************\
  XNSNET 2005 - 2007 Автор: Иванов M.A. [Shum.er] 2005 - 2007 Версия кода: 069
\******************************************************************************/
namespace XNSNET.Collections {

	using System;

	/// <summary>
	/// Объектный класс элемента связанного списка.
	/// </summary>
	public abstract class ObjectedItem {
	
		private ObjectedList mList;
		private ObjectedItem mPrev;
		private ObjectedItem mNext;

		protected ObjectedItem() {
			Invalidate();
		}

		/// <summary>
		/// Возвращает экземпляр списка владеющего этим элементом.
		/// </summary>
		public ObjectedList List {
			get {
				return mList;
			}
			internal set {
				if ( mList != null || value == null )
					throw new ArgumentNullException();
				mList = value;
			}
		}

		/// <summary>
		/// Возвращает приведущий элемент в связанном списке.
		/// </summary>
		public ObjectedItem Prev {
			get {
				return mPrev;
			}
			internal set {
				if ( mPrev == null && value == null )
					throw new ArgumentNullException();
				mPrev = value;
			}
		}

		/// <summary>
		/// Возвращает следующий элемент в связанном списке.
		/// </summary>
		public ObjectedItem Next {
			get {
				return mNext;
			}
			internal set {
				if ( mNext == null && value == null )
					throw new ArgumentNullException();
				mNext = value;
			}
		}

		internal void Invalidate() {
			mList = null;
			mPrev = null;
			mNext = null;
		}

		public bool IsInvalid() {
			return ( mList == null && mPrev == null && mNext == null );
		}

		/// <summary> Удаляет этот элемент из связанного списка. </summary>
		/// <returns> Возвращает собственный экземпляр в случае успеха. </returns>
		public virtual ObjectedItem Remove() {
			if ( mList != null ) {
				return mList.Remove( this );
			}
			return null;
		}

		/// <summary> Событие о добавлении этого элемента в связанный список. </summary>
		/// <param name="iList"> Список в который был добавлен этот элемент. </param>
		public virtual void OnAdded( ObjectedList iList ) {
			iList.RaiseAdded( this );
		}

		/// <summary> Событие о удалении этого элемента из связанного списока. </summary>
		/// <param name="iList"> Список в который был добавлен этот элемент. </param>
		public virtual void OnRemoved( ObjectedList iList ) {
			iList.RaiseRemoved( this );
		}
		
	}

}