﻿/******************************************************************************\
  XNSNET 2005 - 2007 Автор: Иванов M.A. [Shum.er] 2005 - 2007 Версия кода: 069
\******************************************************************************/
namespace XNSNET.Collections {

	using System;

	/// <summary>
	/// Объектный класс списка связанных элементов.
	/// </summary>
	public abstract class ObjectedList {
	
    private int mCount;
    private ObjectedItem mFirst;
    private ObjectedItem mFinal;

		public delegate void Handler( ObjectedList iList, ObjectedItem iItem );

		private event Handler mAdded;

		private event Handler mRemoved;

		public event Handler Added {
			add {
				mAdded += value;
			}
			remove {
				mAdded -= value;
			}
		}

		public event Handler Removed {
			add {
				mRemoved += value;
			}
			remove {
				mRemoved -= value;
			}
		}

		internal void RaiseAdded( ObjectedItem iItem ) {
			if ( mAdded != null )
				mAdded( this, iItem );
		}

		internal void RaiseRemoved( ObjectedItem iItem ) {
			if ( mRemoved != null )
				mRemoved( this, iItem );
		}

		protected ObjectedList() {
      Clearing();
    }

    private void Clearing() {
      mCount = 0;
      mFirst = null;
      mFinal = null;
    }

		public bool IsCleared() {
			return ( mCount == 0 && mFirst == null && mFinal == null );
		}

    public int Count {
      get {
        return mCount;
      }
      private set {
        if ( value < 1 || value == mCount )
          throw new ArgumentOutOfRangeException();
        mCount = value;
      }
    }

		public ObjectedItem First {
      get {
        return mFirst;
      }
      private set {
        if ( value == null || value == mFirst )
          throw new ArgumentNullException();
				mFirst = value;
      }
    }

		public ObjectedItem Final {
      get {
        return mFinal;
      }
			private set {
        if ( value == null || value == mFinal )
          throw new ArgumentNullException();
				mFinal = value;
      }
    }

		public virtual void Clear() {
			if ( !IsCleared() ) {
				lock ( this ) {
					ObjectedItem _Next = this.First;
					ObjectedItem _Item;
					while ( _Next != null ) {
						_Item = _Next;
						_Next = _Next.Next;
						_Item.Invalidate();
						_Item.OnRemoved( this );
					}
					this.Clearing();
				}
      }
    }

		protected ObjectedItem AddFinal( ObjectedItem iItem ) {
			
      if ( iItem == null )
        throw new ArgumentNullException();
      if ( !iItem.IsInvalid() )
        throw new ArgumentException();
      lock ( this ) {
				iItem.List = this;
        if ( this.Count == 0 ) {
          this.First = iItem;
        } else {
					iItem.Prev = this.Final;
					this.Final.Next = iItem;
        }
        this.Final = iItem;
        this.Count++;
				iItem.OnAdded( this );
				return iItem;
      }
    }

		protected ObjectedItem AddFirst( ObjectedItem iItem ) {
			
      if ( iItem == null )
        throw new ArgumentNullException();
      if ( !iItem.IsInvalid() )
        throw new ArgumentException();
      lock ( this ) {
				iItem.List = this;
        if ( this.Count == 0 ) {
          this.Final = iItem;
        } else {
					iItem.Next = this.First;
					this.First.Prev = iItem;
        }
        this.First = iItem;
        this.Count++;
				iItem.OnAdded( this );
				return iItem;
      }
    }

		protected ObjectedItem AddAfter( ObjectedItem iTodo, ObjectedItem iItem ) {
			
      if ( iTodo == null || iItem == null )
        throw new ArgumentNullException();
      if ( iTodo.IsInvalid() || !iItem.IsInvalid() )
        throw new ArgumentException();
      lock ( this ) {
				iItem.List = this;
        iItem.Prev = iTodo;
        if ( iTodo.Next == null ) {
          this.Final = iItem;
        } else {
          iItem.Next = iTodo.Next;
					iTodo.Next.Prev = iItem;
        }
        iTodo.Next = iItem;
        this.Count++;
				iItem.OnAdded( this );
				return iItem;
      }
    }

		protected ObjectedItem AddBefore( ObjectedItem iTodo, ObjectedItem iItem ) {
			
      if ( iTodo == null || iItem == null )
        throw new ArgumentNullException();
      if ( iTodo.IsInvalid() || !iItem.IsInvalid() )
        throw new ArgumentException();
      lock ( this ) {
				iItem.List = this;
        iItem.Next = iTodo;
        if ( iTodo.Prev == null ) {
          this.First = iItem;
        } else {
          iItem.Next = iTodo;
          iTodo.Prev.Next = iItem;
        }
        iTodo.Prev = iItem;
        this.Count++;
				iItem.OnAdded( this );
				return iItem;
      }
    }

		public virtual ObjectedItem Remove( ObjectedItem iItem ) {

      if ( iItem == null )
        throw new ArgumentNullException();

			lock ( this ) {
				if ( this.Count == 1 ) {
					this.Clearing();
				} else {
					if ( iItem.Prev != null && iItem.Next != null ) {
						iItem.Prev.Next = iItem.Next;
						iItem.Next.Prev = iItem.Prev;
					} else if ( iItem.Prev == null ) {
						this.First = iItem.Next;
						iItem.Next.Prev = null;
					} else if ( iItem.Next == null ) {
						this.Final = iItem.Prev;
						iItem.Prev.Next = null;
					}
					Count--;
				}
				iItem.Invalidate();
				iItem.OnRemoved( this );
				return iItem;
			}

    }

		public ObjectedItem[] Array {
			get {
				lock ( this ) {
					ObjectedItem[] _Array = new ObjectedItem[ Count ];
					int _Index = 0;
					ObjectedItem _Item = this.First;
					while ( _Item != null ) {
						_Array[ _Index ] = _Item;
						_Item = _Item.Next;
						_Index++;
					}
					return _Array;
				}
			}
		}
		
	}

}