﻿/******************************************************************************\
  XNSNET 2005 - 2007 Автор: Иванов M.A. [Shum.er] 2005 - 2007 Версия кода: 069
\******************************************************************************/
namespace XNSNET.Collections {

	using System;

	/// <summary> Шаблонный класс элемента связанного списка. </summary>
	/// <typeparam name="LIST"> Шаблонный тип класса списка связанных элементов. </typeparam>
	/// <typeparam name="ITEM"> Шаблонный тип класса элемента связанного списка. </typeparam>
	public abstract class TemplatedItem<LIST, ITEM>
		where LIST: TemplatedList<LIST, ITEM>
		where ITEM: TemplatedItem<LIST, ITEM> {

		private LIST mList;
		private ITEM mPrev;
		private ITEM mNext;
		
		protected TemplatedItem() {
			Invalidate();
		}

		/// <summary> Возвращает экземпляр списка владеющего этим элементом. </summary>
		public LIST List {
			get {
				return mList;
			}
			internal set {
				if ( mList != null || value == null )
					throw new ArgumentNullException();
				mList = value;
			}
		}

		/// <summary> Возвращает приведущий элемент в связанном списке. </summary>
		public ITEM Prev {
			get {
				return mPrev;
			}
			internal set {
				if ( mPrev == null && value == null )
					throw new ArgumentNullException();
				mPrev = value;
			}
		}

		/// <summary> Возвращает следующий элемент в связанном списке. </summary>
		public ITEM Next {
			get {
				return mNext;
			}
			internal set {
				if ( mNext == null && value == null )
					throw new ArgumentNullException();
				mNext = value;
			}
		}

		internal void Invalidate() {
			mList = null;
			mPrev = null;
			mNext = null;
		}

		internal bool IsInvalid() {
			return ( mList == null && mPrev == null && mNext == null );
		}

		/// <summary> Удаляет этот элемент из связанного списка. </summary>
		/// <returns> Возвращает собственный экземпляр в случае успеха. </returns>
		public virtual ITEM Remove() {
			if ( mList != null ) {
				return mList.Remove( (ITEM)this );
			}
			return null;
		}

		/// <summary> Событие о добавлении этого элемента в связанный список. </summary>
		/// <param name="iList"> Список в который был добавлен этот элемент. </param>
		public virtual void OnAdded( LIST iList ) {
			iList.RaiseAdded( (ITEM)this );
		}

		/// <summary> Событие о удалении этого элемента из связанного списока. </summary>
		/// <param name="iList"> Список в который был добавлен этот элемент. </param>
		public virtual void OnRemoved( LIST iList ) {
			iList.RaiseRemoved( (ITEM)this );
		}
		
	}

}