﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class ContextCollection: TemplatedList<ContextCollection, Context> {

		internal ContextCollection(): base() {}

		public Context this[ IntPtr iHandle ] {
			get {
				Context _Next = this.First;
				Context _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item.Handle == iHandle )
						return _Item;
				}
				return null;
			}
		}

		public Context Append( IntPtr iHandle ) {
			lock( this ) {
				if ( iHandle == IntPtr.Zero )
					throw new ArgumentNullException();
				Context _Context = this[ iHandle ];
				if ( _Context != null ) return _Context;
				return AddFinal( new Context( iHandle ) );
			}
		}

		public Context Remove( IntPtr iHandle ) {
			lock( this ) {
				if ( iHandle == IntPtr.Zero )
					throw new ArgumentNullException();
				Context _Context = this[ iHandle ];
				if ( _Context != null ) {
					Remove( _Context );
				}
				return _Context;
			}
		}

	}

	public class Context: TemplatedItem<ContextCollection, Context> {

		private IntPtr mHandle;

		public IntPtr Handle {
			get {
				return mHandle;
			}
		}
		
		private Claster mClaster;
		
		public Claster Claster {
			get {
				return mClaster;
			}
			internal set {
				mClaster = value;
			}
		}

		internal Context( IntPtr iHandle )
			: base() {
			mHandle = iHandle;
			mClaster = null;
		}

		public override void OnRemoved( ContextCollection iList ) {
			if ( mClaster != null ) {
				mClaster.List.Remove( this );
			}
		}

	}

}