using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using XNSNET.Collections;

namespace MetaTrader.Windows {
	public partial class FormClasters: Form {
		
		internal class ListViewClaster: ListViewItem {
			
			private Claster mClaster;
			public Claster Claster {
				get {
					return mClaster;
				}
			}
			
			public ListViewClaster( Claster iClaster ): base( iClaster.Symbol, 0 ) {
				mClaster = iClaster;
				string up = iClaster.Up != null ? iClaster.Up.ToString() : "";
				string down = iClaster.Down != null ? iClaster.Down.ToString() : "";
				if ( iClaster.Ticks.Count > 0 ) {
					ClasterTick _Now = iClaster.Ticks.Final;
					if ( _Now is ClasterTickValue ) {
						this.SubItems.AddRange( new string[] { up, down, iClaster.Session.Count.ToString(), iClaster.Ticks.Count.ToString( "###.###" ), _Now.Bid.ToString(), _Now.Ask.ToString(), _Now.CalledStamp.ToString(), _Now.ServerStamp.ToString() } );
					} else if ( _Now is ClasterTickSpace ) {
						this.SubItems.AddRange( new string[] { up, down, iClaster.Session.Count.ToString(), iClaster.Ticks.Count.ToString( "###.###" ), "", "", _Now.CalledStamp.ToString(), "" } );
					}
				} else {
					this.SubItems.AddRange( new string[] { up, down, iClaster.Session.Count.ToString(), iClaster.Ticks.Count.ToString( "###.###" ), "", "", "", "" } );
				}
				iClaster.Session.Added += new TemplatedList<SessionCollection, Session>.Handler( Session_Added );
				iClaster.Session.Removed += new TemplatedList<SessionCollection, Session>.Handler( Session_Removed );
				iClaster.Ticks.Added += new TemplatedList<ClasterTickCollection,ClasterTick>.Handler(Ticks_Added);
				iClaster.Ticks.Removed += new TemplatedList<ClasterTickCollection, ClasterTick>.Handler( Ticks_Removed );
			}

			void Session_Removed( SessionCollection iList, Session iItem ) {
				if ( this.ListView == null ) return;
				try {
					this.ListView.Parent.Invoke( new TemplatedList<SessionCollection, Session>.Handler( Session ), iList, iItem );
				} catch( ObjectDisposedException ) {}
			}
			void Session_Added( SessionCollection iList, Session iItem ) {
				if ( this.ListView == null ) return;
				try {
					this.ListView.Parent.Invoke( new TemplatedList<SessionCollection, Session>.Handler( Session ), iList, iItem );
				} catch( ObjectDisposedException ) {}
			}

			void Session( SessionCollection iList, Session iItem ) {
				this.SubItems[ 3 ].Text = Claster.Session.Count.ToString();
			}

			void Ticks_Removed( ClasterTickCollection iList, ClasterTick iItem ) {
				if ( this.ListView == null ) return;
				try {
					this.ListView.Parent.Invoke( new TemplatedList<ClasterTickCollection, ClasterTick>.Handler( Ticks ), iList, iItem );
				} catch ( ObjectDisposedException ) { }
			}
			void Ticks_Added( ClasterTickCollection iList, ClasterTick iItem ) {
				if ( this.ListView == null ) return;
				try {
					this.ListView.Parent.Invoke( new TemplatedList<ClasterTickCollection, ClasterTick>.Handler( Ticks ), iList, iItem );
				} catch( ObjectDisposedException ) {}
			}
			
			void Ticks( ClasterTickCollection iList, ClasterTick iItem ) {
				this.SubItems[ 4 ].Text = iList.Count.ToString( "###.###" );
				if ( iItem is ClasterTickValue ) {
					this.SubItems[ 5 ].Text = iItem.Bid.ToString();
					this.SubItems[ 6 ].Text = iItem.Ask.ToString();
					this.SubItems[ 7 ].Text = iItem.CalledStamp.ToString();
					this.SubItems[ 8 ].Text = iItem.ServerStamp.ToString();
				} else if ( iItem is ClasterTickSpace ) {
					this.SubItems[ 5 ].Text = "";
					this.SubItems[ 6 ].Text = "";
					this.SubItems[ 7 ].Text = iItem.CalledStamp.ToString();
					this.SubItems[ 8 ].Text = "";
				}
			}

			public override void Remove() {
				ClasterTickCollection ticks = Claster.Ticks;
					ticks.Added -= Ticks_Added;
					ticks.Removed -= Ticks_Removed;
				SessionCollection sessions = Claster.Session;
					sessions.Added -= Session_Added;
					sessions.Removed -= Session_Removed;
				base.Remove();
			}
			
		}
		
		public FormClasters() {
			InitializeComponent();
			foreach ( Claster item in Global.Claster.Array ) {
				SymbolList.Items.Add( new ListViewClaster( item ) );
			}
			Global.Claster.Added += new TemplatedList<ClasterCollection, Claster>.Handler( Claster_Added );
			Global.Claster.Removed += new TemplatedList<ClasterCollection, Claster>.Handler( Claster_Removed );
		}

		void Claster_Removed( ClasterCollection iList, Claster iItem ) {
			try {
				Invoke( new TemplatedList<ClasterCollection, Claster>.Handler( ClasterRemoved ), iList, iItem );
			} catch( ObjectDisposedException ) {}
		}

		void Claster_Added( ClasterCollection iList, Claster iItem ) {
			try {
				Invoke( new TemplatedList<ClasterCollection, Claster>.Handler( ClasterAdded ), iList, iItem );
			} catch ( ObjectDisposedException ) { }
		}

		void ClasterRemoved( ClasterCollection iList, Claster iItem ) {
			ListViewItem[] items = new ListViewItem[ SymbolList.Items.Count ];
			SymbolList.Items.CopyTo( items, 0 );
			foreach ( ListViewItem item in items ) {
				ListViewClaster claster = item as ListViewClaster;
				if ( claster != null && claster.Claster == iItem ) {
					claster.Remove();
				}
			}
		}

		void ClasterAdded( ClasterCollection iList, Claster iItem ) {
			SymbolList.Items.Add( new ListViewClaster( iItem ) );
		}

		private void WindowClasters_Load( object sender, EventArgs e ) {
			
		}

		protected override void Dispose( bool disposing ) {
			if ( disposing  ) {
				ListViewItem[] items = new ListViewItem[ SymbolList.Items.Count ];
				SymbolList.Items.CopyTo( items, 0 );
				foreach ( ListViewItem item in items ) {
					ListViewClaster claster = item as ListViewClaster;
					if ( claster != null ) {
						claster.Remove();
					}
				}
				if ( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}
		
	}
	
}