using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using XNSNET.Collections;

namespace MetaTrader.Windows {
	public partial class FormDiagram: Form {

		private Claster mClaster;

		public Claster Claster {
			get {
				return mClaster;
			}
		}

		private int OldWidth;
		private int OldHeight;
		private int NowWidth;
		private int NowHeight;
		private Bitmap WorkBuffer;
		private Graphics WorkOutput;
		
		public FormDiagram( Claster iClaster ) {
			mClaster = iClaster;
			InitializeComponent();
			NowWidth = OldWidth = ClientSize.Width;
			NowHeight = OldHeight = ClientSize.Height;
			WorkBuffer = new Bitmap( OldWidth, OldHeight );
			WorkOutput = Graphics.FromImage( WorkBuffer );
			Text = iClaster.Symbol;
			FloatLine = NowHeight / 2;
			FloatPrice = 0;
			Claster.Ticks.Added += Ticks_Added;
		}

		void Ticks_Added( ClasterTickCollection iList, ClasterTick iItem ) {
			InvokeRedrawContent();
		}

		protected override void Dispose( bool disposing ) {
			if ( disposing ) {
				Claster.Ticks.Added -= Ticks_Added;
				WorkBuffer.Dispose();
				WorkBuffer = null;
				WorkOutput.Dispose();
				WorkOutput = null;
				mClaster = null;
				if ( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		protected override void OnResize( EventArgs e ) {
			try {
				if ( ClientSize.Width > OldWidth || ClientSize.Height > OldHeight ) {
					if ( ClientSize.Width > OldWidth ) OldWidth = ClientSize.Width;
					if ( ClientSize.Height > OldHeight ) OldHeight = ClientSize.Height;
					if ( WorkBuffer != null ) WorkBuffer.Dispose();
					WorkBuffer = new Bitmap( OldWidth, OldHeight );
					if ( WorkOutput != null ) WorkOutput.Dispose();
					WorkOutput = Graphics.FromImage( WorkBuffer );
				}
				NowWidth = ClientSize.Width - 1; NowHeight = ClientSize.Height - 1;
				WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
				OnDrawContent( WorkOutput, NowWidth, NowHeight );
				using( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
					DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
				}
				FloatLine = NowHeight / 2;
				FloatPrice = 0;
			} catch {}
			base.OnResize( e );
		}

		protected override void OnLoad( EventArgs e ) {
			NowWidth = ClientSize.Width - 1; NowHeight = ClientSize.Height - 1;
			WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
			OnDrawContent( WorkOutput, NowWidth, NowHeight );
			using ( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
			base.OnLoad( e );
		}

		protected override void OnShown( EventArgs e ) {
			NowWidth = ClientSize.Width - 1; NowHeight = ClientSize.Height - 1;
			WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
			OnDrawContent( WorkOutput, NowWidth, NowHeight );
			using ( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
			base.OnShown( e );
		}

		protected override void OnPaintBackground( PaintEventArgs e ) {
			e.Graphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
		}

		public virtual void RedrawContent() {
			using ( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
				OnDrawContent( WorkOutput, NowWidth, NowHeight );
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
		}
		
		private delegate void ReadrawCallback();

		public void InvokeRedrawContent() {
			try {
				Invoke( new ReadrawCallback( RedrawContent ) );
			} catch ( ObjectDisposedException ) {}
		}

		public void ResetRedrawContent() {
			FloatLine = NowHeight / 2;
			FloatPrice = 0;
			RedrawContent();
		}

		double FloatLine;
		double FloatPrice;
		
		protected virtual void OnDrawContent( Graphics iGraphics, int iWidth, int iHeight ) {

			if ( Claster.Ticks.Count == 0 ) return;

			double _IndexX = iWidth - 10;
			double _Point;
			double _Price;
			
			ClasterTick _Next = Claster.Ticks.Final;
			ClasterTick _Prev = _Next.Prev;
			if ( _Next != null && _Next is ClasterTickValue ) {
				_Price = _Next.BidInt * Claster.DiagramTickHeight;
			} else if ( _Prev != null && _Prev is ClasterTickValue ) {
				_Price = _Prev.BidInt * Claster.DiagramTickHeight;
			} else {
				return;
			}
			if ( FloatPrice == 0 ) {
				FloatPrice = _Price;
			} else if ( FloatPrice > _Price ) {
				_Point = FloatPrice - _Price;
				FloatLine += _Point;
				if ( FloatLine > NowHeight ) {
					_Point = FloatLine - NowHeight;
					FloatLine = NowHeight;
					FloatPrice -= _Point;
				} else {
					FloatPrice -= _Point;
				}
			} else if ( FloatPrice < _Price ) {
				_Point = _Price - FloatPrice;
				FloatLine -= _Point;
				if ( FloatLine < 0 ) {
					_Point = 0 - FloatLine;
					FloatLine = 0;
					FloatPrice += _Point;
				} else {
					FloatPrice += _Point;
				}
			}

			iGraphics.DrawLine( Pens.Green, 0, (int)( FloatLine ), iWidth, (int)( FloatLine ) );
			
			while ( _Prev != null && _IndexX > 0 ) {
				if ( _Next is ClasterTickSpace ) {
					iGraphics.DrawLine( Pens.Gray, (int)( _IndexX ), 0, (int)( _IndexX ), (int)( iHeight ) );
					_Next = _Prev;
					_Prev = _Prev.Prev;
					_IndexX--;
				} else if ( _Prev is ClasterTickSpace ) {
					iGraphics.DrawLine( Pens.Gray, (int)( _IndexX ), 0, (int)( _IndexX ), (int)( iHeight ) );
					_Prev = _Prev.Prev;
					_IndexX--;
				} else {
					double _PriceNext = FloatLine - ( ( _Next.BidInt * Claster.DiagramTickHeight ) - FloatPrice );
					double _PricePrev = FloatLine - ( ( _Prev.BidInt * Claster.DiagramTickHeight ) - FloatPrice );
					if ( _PriceNext < _PricePrev ) {
						iGraphics.DrawLine( Pens.Lime, (int)( _IndexX - Claster.DiagramTickWidth ), (int)( _PricePrev ), (int)( _IndexX ), (int)( _PriceNext ) );
					} else {
						iGraphics.DrawLine( Pens.Red, (int)( _IndexX - Claster.DiagramTickWidth ), (int)( _PricePrev ), (int)( _IndexX ), (int)( _PriceNext ) );
					}
					_Next = _Prev;
					_Prev = _Prev.Prev;
					_IndexX -= Claster.DiagramTickWidth;
				}
			}
			
		}

		private void MenuProperties_Click( object sender, EventArgs e ) {
			new FormClasterProperties( Claster, this ).ShowDialog( this );
		}

	}
}